@@ -4,7 +4,7 @@ use strict;
 use vars qw($VERSION);
 use Carp;
 
-$VERSION = '2.05';
+$VERSION = '2.07';
 
 
 # LOAD FILTERING MODULE...
@@ -22,7 +22,6 @@ my ($Perl5, $Perl6) = (0,0);
 
 sub import
 {
-	$DB::single = 1;
 	$fallthrough = grep /\bfallthrough\b/, @_;
 	$offset = (caller)[2]+1;
 	filter_add({}) unless @_>1 && $_[1] eq 'noimport';
@@ -74,6 +73,16 @@ sub is_block
 	return !$ishash;
 }
 
+
+my $EOP = qr/\n\n|\Z/;
+my $CUT = qr/\n=cut.*$EOP/;
+my $pod_or_DATA = qr/ ^=(?:head[1-4]|item) .*? $CUT
+                    | ^=pod .*? $CUT
+                    | ^=for .*? $EOP
+                    | ^=begin \s* (\S+) .*? \n=end \s* \1 .*? $EOP
+                    | ^__(DATA|END)__\n.*
+                    /smx;
+
 my $casecounter = 1;
 sub filter_blocks
 {
@@ -82,6 +91,7 @@ sub filter_blocks
 			   || $Perl6 && $source =~ /when|given/;
 	pos $source = 0;
 	my $text = "";
+	$DB::single = 1;
 	component: while (pos $source < length $source)
 	{
 		if ($source =~ m/(\G\s*use\s+Switch\b)/gc)
@@ -89,12 +99,15 @@ sub filter_blocks
 			$text .= q{use Switch 'noimport'};
 			next component;
 		}
-		my @pos = Text::Balanced::_match_quotelike(\$source,qr/\s*/,1,1);
+		my @pos = Text::Balanced::_match_quotelike(\$source,qr/\s*/,1,0);
 		if (defined $pos[0])
 		{
 			$text .= " " . substr($source,$pos[2],$pos[18]-$pos[2]);
 			next component;
 		}
+		if ($source =~ m/\G\s*($pod_or_DATA)/gc) {
+			next component;
+		}
 		@pos = Text::Balanced::_match_variable(\$source,qr/\s*/);
 		if (defined $pos[0])
 		{
@@ -103,15 +116,20 @@ sub filter_blocks
 		}
 
 		if ($Perl5 && $source =~ m/\G(\n*)(\s*)(switch)\b(?=\s*[(])/gc
-		 || $Perl6 && $source =~ m/\G(\n*)(\s*)(given)\b(?=\s*[(])/gc)
+		 || $Perl6 && $source =~ m/\G(\n*)(\s*)(given)\b(?=\s*[(])/gc
+		 || $Perl6 && $source =~ m/\G(\n*)(\s*)(given)\b(.*)(?=\{)/gc)
 		{
 			my $keyword = $3;
+			my $arg = $4;
+			print  STDERR "[$arg]\n";
 			$text .= $1.$2.'S_W_I_T_C_H: while (1) ';
-			@pos = Text::Balanced::_match_codeblock(\$source,qr/\s*/,qr/\(/,qr/\)/,qr/[[{(<]/,qr/[]})>]/,undef) 
-			or do {
-				die "Bad $keyword statement (problem in the parentheses?) near $Switch::file line ", line(substr($source,0,pos $source),$line), "\n";
-			};
-			my $arg = filter_blocks(substr($source,$pos[0],$pos[4]-$pos[0]),line(substr($source,0,$pos[0]),$line));
+			unless ($arg) {
+				@pos = Text::Balanced::_match_codeblock(\$source,qr/\s*/,qr/\(/,qr/\)/,qr/[[{(<]/,qr/[]})>]/,undef) 
+				or do {
+					die "Bad $keyword statement (problem in the parentheses?) near $Switch::file line ", line(substr($source,0,pos $source),$line), "\n";
+				};
+				$arg = filter_blocks(substr($source,$pos[0],$pos[4]-$pos[0]),line(substr($source,0,$pos[0]),$line));
+			}
 			$arg =~ s {^\s*[(]\s*%}   { ( \\\%}	||
 			$arg =~ s {^\s*[(]\s*m\b} { ( qr}	||
 			$arg =~ s {^\s*[(]\s*/}   { ( qr/}	||
@@ -149,7 +167,7 @@ sub filter_blocks
 				$code =~ s {^\s*@}  { \@};
 				$text .= " $code)";
 			}
-			elsif ( @pos = Text::Balanced::_match_quotelike(\$source,qr/\s*/,1,1)) {
+			elsif ( @pos = Text::Balanced::_match_quotelike(\$source,qr/\s*/,1,0)) {
 				my $code = substr($source,$pos[2],$pos[18]-$pos[2]);
 				$code = filter_blocks($code,line(substr($source,0,$pos[2]),$line));
 				$code =~ s {^\s*m}  { qr}	||
@@ -158,7 +176,7 @@ sub filter_blocks
 				$text .= " $code)";
 			}
 			elsif ($Perl5 && $source =~ m/\G\s*(([^\$\@{])[^\$\@{]*)(?=\s*{)/gc
-			   ||  $Perl6 && $source =~ m/\G\s*([^:;]*)()/gc) {
+			   ||  $Perl6 && $source =~ m/\G\s*([^;{]*)()/gc) {
 				my $code = filter_blocks($1,line(substr($source,0,pos $source),$line));
 				$text .= ' \\' if $2 eq '%';
 				$text .= " $code)";
@@ -167,8 +185,8 @@ sub filter_blocks
 				die "Bad $keyword statement (invalid $keyword value?) near $Switch::file line ", line(substr($source,0,pos $source), $line), "\n";
 			}
 
-		        die "Missing colon or semi-colon after 'when' value near $Switch::file line ", line(substr($source,0,pos $source), $line), "\n"
-				unless !$Perl6 || $source =~ m/\G(\s*)(:|(?=;))/gc;
+		        die "Missing opening brace or semi-colon after 'when' value near $Switch::file line ", line(substr($source,0,pos $source), $line), "\n"
+				unless !$Perl6 || $source =~ m/\G(\s*)(?=;|\{)/gc;
 
 			do{@pos = Text::Balanced::_match_codeblock(\$source,qr/\s*/,qr/\{/,qr/\}/,qr/\{/,qr/\}/,undef)}
 			or do {
@@ -186,7 +204,7 @@ sub filter_blocks
 			next component;
 		}
 
-		$source =~ m/\G(\s*(\w+|#.*\n|\W))/gc;
+		$source =~ m/\G(\s*(-[sm]\s+|\w+|#.*\n|\W))/gc;
 		$text .= $1;
 	}
 	$text;
@@ -341,7 +359,8 @@ sub switch(;$)
 	return 1;
 }
 
-sub case($) { $::_S_W_I_T_C_H->(@_); }
+sub case($) { local $SIG{__WARN__} = \&carp;
+	      $::_S_W_I_T_C_H->(@_); }
 
 # IMPLEMENT __
 
@@ -473,8 +492,8 @@ Switch - A switch statement for Perl
 
 =head1 VERSION
 
-This document describes version 2.05 of Switch,
-released September  3, 2001.
+This document describes version 2.07 of Switch,
+released May 15, 2002.
 
 =head1 SYNOPSIS
 
@@ -725,23 +744,25 @@ Perl 6 will provide a built-in switch statement with essentially the
 same semantics as those offered by Switch.pm, but with a different
 pair of keywords. In Perl 6 C<switch> with be spelled C<given>, and
 C<case> will be pronounced C<when>. In addition, the C<when> statement
-will use a colon between its case value and its block (removing the
-need to parenthesize variables.
+will not require switch or case values to be parenthesized.
 
-This future syntax is also available via the Switch.pm module, by
+This future syntax is also (largely) available via the Switch.pm module, by
 importing it with the argument C<"Perl6">.  For example:
 
         use Switch 'Perl6';
 
         given ($val) {
-                when 1 :      { handle_num_1(); }
-                when $str1 :  { handle_str_1(); }
-                when [0..9] : { handle_num_any(); last }
-                when /\d/ :   { handle_dig_any(); }
-                when /.*/ :   { handle_str_any(); }
+                when 1       { handle_num_1(); }
+                when ($str1) { handle_str_1(); }
+                when [0..9]  { handle_num_any(); last }
+                when /\d/    { handle_dig_any(); }
+                when /.*/    { handle_str_any(); }
         }
 
-Note that you can mix and match both syntaxes by importing the module
+Note that scalars still need to be parenthesized, since they would be
+ambiguous in Perl 5.
+
+Note too that you can mix and match both syntaxes by importing the module
 with:
 
 	use Switch 'Perl5', 'Perl6';
@@ -825,6 +846,12 @@ Damian Conway (damian@conway.org)
 There are undoubtedly serious bugs lurking somewhere in code this funky :-)
 Bug reports and other feedback are most welcome.
 
+=head1 LIMITATION
+
+Due to the heuristic nature of Switch.pm's source parsing, the presence
+of regexes specified with raw C<?...?> delimiters may cause mysterious
+errors. The workaround is to use C<m?...?> instead.
+
 =head1 COPYRIGHT
 
     Copyright (c) 1997-2001, Damian Conway. All Rights Reserved.
